<?php

namespace Drupal\simplenews_stats\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\simplenews_stats\SimplenewsStatsPage;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\simplenews_stats\SimplenewsStatsEngine;
use Drupal\simplenews_stats\SimplenewsStatsAllowedLinks;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Url;

/**
 * Provides route responses for hits and stats page.
 */
class SimplenewsStatsController extends ControllerBase {

  /**
   * SimplenewsStatsEngine.
   *
   * @var \Drupal\simplenews_stats\SimplenewsStatsEngine
   */
  protected $simplenewsStatsEngine;

  /**
   * SimplenewsStatsAllowedLinks.
   *
   * @var \Drupal\simplenews_stats\SimplenewsStatsAllowedLinks
   */
  protected $simplenewsStatsAllowedLinks;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * SimplenewsStatsController constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The request stack.
   * @param \Drupal\simplenews_stats\SimplenewsStatsEngine $simplenewsStatsEngine
   *   The simplenews stats engine.
   * @param \Drupal\simplenews_stats\SimplenewsStatsAllowedLinks $simplenewsStatsAllowedLinks
   *   The simplenews stats Allowed links manager.
   */
  public function __construct(RequestStack $request, SimplenewsStatsEngine $simplenewsStatsEngine, SimplenewsStatsAllowedLinks $simplenewsStatsAllowedLinks) {
    $this->request                     = $request;
    $this->simplenewsStatsEngine       = $simplenewsStatsEngine;
    $this->simplenewsStatsAllowedLinks = $simplenewsStatsAllowedLinks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('simplenews_stats.engine'),
      $container->get('simplenews_stats.allowedlinks')
    );
  }

  /**
   * Access callback: Check if the node is a simplenews and if the user has
   * access.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The account.
   * @param \Drupal\node\NodeInterface $node
   *   The node object.
   *
   * @return \Drupal\Core\Access\AccessResult
   */
  public static function simplenewsStatsAccess(AccountInterface $account, NodeInterface $node) {
    if ($node->hasField('simplenews_issue') && !$node->get('simplenews_issue')->isEmpty()) {
      if ($account->hasPermission('access simplenews stats results')) {
        return AccessResult::allowed();
      }
      elseif ($account->hasPermission('access simplenews stats results editable node') && $node->access('update', $account)) {
        return AccessResult::allowed()->addCacheableDependency($node);
      }
    }

    return AccessResult::neutral();
  }


  /**
   * Send image.
   */
  public function hitView() {
    $response = new Response();
    $image    = file_get_contents(drupal_get_path('module', 'simplenews_stats') . '/image/simple.png');
    $response->setContent($image);
    $response->headers->set('Content-Type', 'image/png');
    $response->headers->set('Content-Transfer-Encoding', 'binary');
    return $response;
  }

  /**
   * Catch click and redirect to link.
   *
   * @param string $tag
   *   The tag.
   */
  public function hitClick($tag) {

    $entities = $this->simplenewsStatsEngine->getTagEntities($tag);
    if ($entities === FALSE) {
      return new RedirectResponse('/');
    }

    // Log click and redirect to the external link if it allowed.
    $link = $this->request->getCurrentRequest()->query->get('link');
    if ($this->simplenewsStatsAllowedLinks->isLinkExist($entities['entity'], $link)) {
      $this->simplenewsStatsEngine->addStatTags($tag, $link);

      // Use TrustedRedirectResponse for this external redirection.
      return new TrustedRedirectResponse(Url::fromUri($link)->toString());
    }

    // Redirect to the entity if the link is not allowed.
    return new RedirectResponse($entities['entity']->toUrl()->toString());
  }

  /**
   * Stats page callback.
   *
   * @param \Drupal\Core\Entity\EntityInterface $node
   *   The node used by simplenews.
   */
  public function stats(EntityInterface $node) {
    $simplenewsStatPage = new SimplenewsStatsPage($node);
    return $simplenewsStatPage->getpage();
  }

}
